from cnvrgv2.config import routes
from cnvrgv2.modules.base.dynamic_attributes import DynamicAttributes
from cnvrgv2.proxy import HTTP, Proxy
from cnvrgv2.context import Context, SCOPE
from cnvrgv2.utils.json_api_format import JAF


class StorageClass(DynamicAttributes):
    available_attributes = {
        "slug": str,
        "title": str,
        "host_path": str,
        "host_ip": str,
        "connection_type": str,
        "status": str,
    }

    def __init__(self, context=None, slug=None, attributes=None):
        super().__init__()
        self._context = Context(context=context)
        if slug:
            self._context.set_scope(SCOPE.STORAGE_CLASS, slug)

        scope = self._context.get_scope(SCOPE.STORAGE_CLASS)

        self._proxy = Proxy(context=self._context)
        self._route = routes.STORAGE_CLASS_BASE.format(scope["organization"], slug)
        self._attributes = attributes or {}
        self.slug = scope["storage_class"]

    def delete(self):
        """
        Deletes the current storage class
        @return: None
        """
        query_param = "?delete_from_cluster=true"
        delete_route = self._route + query_param
        self._proxy.call_api(route=delete_route, http_method=HTTP.DELETE)

    def disconnect(self):
        """
        Disconnect the current storage class
        @return: None
        """
        self._proxy.call_api(route=self._route, http_method=HTTP.DELETE,
                             payload=JAF.serialize(type="storageClass", attributes={}))



from cnvrgv2.config import routes
from cnvrgv2.modules.base.dynamic_attributes import DynamicAttributes
from cnvrgv2.proxy import Proxy, HTTP
from cnvrgv2.context import Context, SCOPE
from cnvrgv2.utils.json_api_format import JAF

STATUS_MAP = {
    "init": "Initializing",
    "created": "Created",
    "up": "Available",
    "removed": "Removed",
    "error": "Error",
    "in_use": "In Use"
}
IN_USE = "in_use"

READ_WRITE_MANY = "Read Write Many"
READ_WRITE_ONCE = "Read Write Once"


class Volume(DynamicAttributes):
    available_attributes = {
        "slug": str,
        "title": str,
        "total_space": int,
        "used_space": int,
        "host_path": str,
        "host_ip": str,
        "read_only": bool,
        "hide": bool,
        "volume_type": str,
        "mount_path": str,
        "claim_name": str,
        "status": str,
        "in_use": bool,
        "using_jobs": list,
        "storage_class_slug": str,
        "access_mode": str
    }

    def __init__(self, context=None, slug=None, attributes=None):
        self._context = Context(context=context)

        # Set current context scope to current volume
        if slug:
            self._context.set_scope(SCOPE.VOLUME, slug)

        scope = self._context.get_scope(SCOPE.VOLUME)

        self._proxy = Proxy(context=self._context)
        self._route = routes.VOLUME_BASE.format(scope["organization"], scope["volume"])
        self._attributes = attributes or {}
        self.slug = scope["volume"]
        self.status = self.get_status()
        self.access_mode = self.get_access_mode()

    def delete(self):
        """
        Deletes the current volume
        @return: None
        """
        delete_route = self._route + "?delete_from_cluster=true"
        self._proxy.call_api(route=delete_route, http_method=HTTP.DELETE)

    def disconnect(self):
        """
        Disconnect the current volume
        @return: None
        """
        self._proxy.call_api(route=self._route, http_method=HTTP.DELETE,
                             payload=JAF.serialize(type="volume", attributes={}))

    def get_access_mode(self):
        return READ_WRITE_ONCE if self.read_only is False else READ_WRITE_MANY

    def get_status(self):
        if self.read_only and self.in_use:
            return STATUS_MAP[IN_USE]
        return STATUS_MAP[self.status]



import time

from cnvrgv2 import cnvrg
from cnvrgv2.modules.resources.clusters_client import ClustersClient
from cnvrgv2.modules.storage_class.storage_class import StorageClass
from cnvrgv2.proxy import Proxy, HTTP
from cnvrgv2.errors import CnvrgArgumentsError
from cnvrgv2.context import Context, SCOPE
from cnvrgv2.config import error_messages, routes
from cnvrgv2.utils.api_list_generator import api_list_generator
from cnvrgv2.utils.json_api_format import JAF
from cnvrgv2.utils.url_utils import urljoin


class StorageClassesClient:
    def __init__(self, organization):
        if type(organization) is not cnvrg.Cnvrg:
            raise CnvrgArgumentsError(error_messages.ARGUMENT_BAD_TYPE.format(cnvrg.Cnvrg, type(organization)))
        self._context = Context(context=organization._context)
        scope = self._context.get_scope(SCOPE.ORGANIZATION)
        self._proxy = Proxy(context=self._context)
        self._route = routes.STORAGE_CLASSES_BASE.format(scope["organization"])

    def list(self, sort="-id", page_size=20, filter=None):
        """
        List all StorageClass in a specific organization
        @param sort: key to sort the list by (-key -> DESC | key -> ASC)
        @param page_size: size of list per page
        @param filter: using filter to show specific results
        @raise: HttpError
        @return: Generator that yields StorageClass objects
        """
        return api_list_generator(
            context=self._context,
            route=self._route,
            object=StorageClass,
            sort=sort,
            filter=filter,
            page_size=page_size
        )

    def get(self, slug=None, name=None):
        """
        Retrieves an StorageClass by the given slug, or name
        @param slug: The slug of the requested StorageClass
        @param name: The name of the requested StroageClass
        @return: StorageClass object
        """

        if slug and isinstance(slug, str):
            return StorageClass(context=self._context, slug=slug)
        elif not slug and name:
            get_storage_class_by_name_url = urljoin(self._route, routes.GET_BY_NAME_SUFFIX)

            attributes = {
                "storage_class_name": name,
            }

            res_attributes = self._proxy.call_api(
                route=get_storage_class_by_name_url,
                http_method=HTTP.GET,
                payload=attributes
            ).attributes

            return StorageClass(context=self._context, slug=res_attributes['slug'], attributes=res_attributes)

        else:
            raise CnvrgArgumentsError(error_messages.STORAGE_CLASS_GET_FAULTY_PARAMS)

    def create(self, title, host_ip, host_path, cluster=None, wait=False):
        """
        Creates a new Storage class
        @param cluster: cluster slug to create storage class on
        @param title: Title of the StorageClass
        @param host_ip: Host IP of NFS
        @param host_path: Host path of NFS
        @param cluster_slug: Cluster slug where the Storage Class will be created.
        @param wait: Wait to Storage Class to be created
        @return:
        """

        cluster = cluster or self.get_default_cluster_slug()
        attributes = {
            "storage_class_title": title,
            "host_ip": host_ip,
            "host_path": host_path,
            "cluster_slug": cluster
        }

        res_attributes = self._proxy.call_api(
            route=self._route,
            http_method=HTTP.POST,
            payload=JAF.serialize(type="StorageClass", attributes=attributes)
        ).attributes
        if wait:
            return self.polling_status(polling_interval=10, slug=res_attributes["slug"])

        return StorageClass(context=self._context, slug=res_attributes["slug"], attributes=res_attributes)

    def connect(self, title, connect_all_volumes=False, cluster=None, wait=False):
        """
        Connect existing Storage Class
        @param title: Title of the storage class in kubernetes
        @param connect_all_volumes: connect all storage class volumes
        @param cluster_slug: Cluster slug where the Storage Class will be created.
        @param wait: Wait to Storage Class to be created
        @return:
        """
        attributes = {
            "storage_class_title_k8s": title,
            "connect_all_volumes_exists": connect_all_volumes,
            "cluster_slug": cluster
        }

        res_attributes = self._proxy.call_api(
            route=self._route,
            http_method=HTTP.POST,
            payload=JAF.serialize(type="StorageClass", attributes=attributes)
        ).attributes
        if wait:
            return self.polling_status(polling_interval=10, slug=res_attributes["slug"])
        return StorageClass(context=self._context, slug=res_attributes["slug"], attributes=res_attributes)

    def polling_status(self, polling_interval=10, slug=None):
        res_attributes = self.get(slug)
        while res_attributes.status == "pending":
            res_attributes = self.get(slug)
            time.sleep(polling_interval)
        return res_attributes

    def get_default_cluster_slug(self):
        cluster = next((item for item in ClustersClient(self).list() if item.is_default), None)
        return cluster.slug



import click
from cnvrgv2.cli.utils import messages
from cnvrgv2.cli.utils.decorators import prepare_command
from cnvrgv2.cli.utils.helpers import print_generator_in_chunks
from cnvrgv2.config import error_messages
from cnvrgv2.errors import CnvrgError
from cnvrgv2.utils.validators import validate_storage_class_title


@click.group(name='storage-class')
def storage_class_group():
    pass


@storage_class_group.command()
@click.option('-t', '--title', prompt=messages.STORAGE_CLASS_PROMPT_TITLE, help=messages.STORAGE_CLASS_HELP_TITLE)
@click.option('-i', '--host-ip', prompt=messages.STORAGE_CLASS_PROMPT_HOST_IP, help=messages.STORAGE_CLASS_HELP_HOST_IP)
@click.option('-p', '--host-path', prompt=messages.STORAGE_CLASS_PROMPT_HOST_PATH,
              help=messages.STORAGE_CLASS_HELP_HOST_PATH)
@click.option('-c', '--cluster-slug', prompt=messages.STORAGE_CLASS_PROMPT_CLUSTER,
              help=messages.STORAGE_CLASS_HELP_CLUSTER)
@click.option('-w', '--wait', default=False, help=messages.STORAGE_CLASS_WAIT)
@prepare_command()
def create(cnvrg, logger, title=None, host_ip=None, host_path=None, cluster_slug=None, wait=False):
    """
    Create Storage Class
    """
    if not validate_storage_class_title(title):
        logger.log_and_echo(error_messages.INVALID_STORAGE_CLASS_TITLE.format(title), error=True)
    logger.log_and_echo(messages.STORAGE_CLASS_CREATING_MESSAGE)
    new_storage_class = cnvrg.storage_classes.create(title=title, host_ip=host_ip,
                                                     host_path=host_path, cluster=cluster_slug, wait=wait)
    success_message = messages.CREATED_SUCCESSFULLY.format(new_storage_class.title, new_storage_class.slug)
    logger.log_and_echo(success_message)


@storage_class_group.command()
@click.option('-t', '--title', prompt=messages.STORAGE_CLASS_PROMPT_CONNECT_TITLE,
              help=messages.STORAGE_CLASS_HELP_CONNECT_TITLE)
@click.option('-cv', '--connect-volumes', default=False, prompt=messages.STORAGE_CLASS_CONNECT_ALL_VOLUMES_PROMPT,
              help=messages.STORAGE_CLASS_CONNECT_ALL_VOLUMES_HELP)
@click.option('-c', '--cluster-slug', prompt=messages.STORAGE_CLASS_PROMPT_CLUSTER,
              help=messages.STORAGE_CLASS_HELP_CLUSTER)
@click.option('-w', '--wait', default=False, help=messages.STORAGE_CLASS_WAIT)
@prepare_command()
def connect(cnvrg, logger, title=None, connect_volumes=False, cluster_slug=None, wait=False):
    """
    Connect Storage Class
    """
    if not validate_storage_class_title(title):
        logger.log_and_echo(error_messages.INVALID_STORAGE_CLASS_TITLE.format(title), error=True)
    logger.log_and_echo(messages.STORAGE_CLASS_CONNECT_MESSAGE)
    connected_storage_class = cnvrg.storage_classes.connect(title=title, connect_all_volumes=connect_volumes,
                                                            cluster=cluster_slug, wait=wait)
    success_message = messages.CREATED_SUCCESSFULLY.format(connected_storage_class.title, connected_storage_class.slug)
    logger.log_and_echo(success_message)


@storage_class_group.command()
@click.option('-s', '--slug', help=messages.STORAGE_CLASS_HELP_SLUG)
@prepare_command()
def disconnect(cnvrg, logger, slug):
    """
    disconnect a storage_class
    """
    try:
        storage_class = cnvrg.storage_classes.get(slug=slug)
    except CnvrgError as error:
        logger.log_and_echo(message=str(error), error=True)
        return
    success_message = messages.DISCONNECT_MESSAGE_SUCCESSFULLY.format(storage_class.title, storage_class.slug)
    storage_class.disconnect()
    logger.log_and_echo(success_message)


@storage_class_group.command()
@click.option('-s', '--slug', help=messages.STORAGE_CLASS_HELP_SLUG)
@prepare_command()
def delete(cnvrg, logger, slug):
    """
    Deletes a storage class
    """
    try:
        storage_class = cnvrg.storage_classes.get(slug=slug)
    except CnvrgError as error:
        logger.log_and_echo(message=str(error), error=True)
        return
    storage_class.delete()
    success_message = messages.DELETE_MESSAGE_SUCCESSFULLY.format(storage_class.title, storage_class.slug)
    logger.log_and_echo(success_message)


@storage_class_group.command(name='get')
@click.option('-s', '--slug', help=messages.STORAGE_CLASS_HELP_SLUG)
@prepare_command()
def get(cnvrg, logger, slug):
    """
    Get a specific storage class by slug
    """
    try:
        storage_class = cnvrg.storage_classes.get(slug=slug)
    except CnvrgError as error:
        logger.log_and_echo(message=str(error), error=True)
        return
    attributes_field_name = ["Title", "Slug", "Host IP", "Host Path", "Status"]
    attributes = ["title", "slug", "host_ip", "host_path", "status"]
    print_generator_in_chunks(
        generator=iter([storage_class]),
        chunk_size=1,
        limit=1,
        object_attributes=attributes,
        attributes_field_name=attributes_field_name,
    )


@storage_class_group.command(name='list')
@click.option('-p', '--page-size', default=20, help=messages.PAGE_SIZE_HELP)
@click.option('-s', '--sort', default="-id", help=messages.SORT_HELP)
@click.option('-cs', '--chunk-size', default=20, type=int, help='Number of storage classes to display at a time')
@click.option('-l', '--limit', type=int, help='Maximum number of storage classes to display')
@prepare_command()
def list_command(cnvrg, logger, sort, chunk_size, limit, page_size):
    """
    List all storage classes
    """
    try:
        storage_classes = cnvrg.storage_classes.list(sort=sort, page_size=page_size)
    except CnvrgError as error:
        logger.log_and_echo(message=str(error), error=True)
        return
    attributes_field_name = ["Title", "Slug", "Host IP", "Host Path", "Status"]
    attributes = ["title", "slug", "host_ip", "host_path", "status"]
    print_generator_in_chunks(
        generator=storage_classes,
        chunk_size=chunk_size,
        limit=limit,
        object_attributes=attributes,
        attributes_field_name=attributes_field_name,
    )



import click
from cnvrgv2.cli.utils import messages
from cnvrgv2.cli.utils.decorators import prepare_command
from cnvrgv2.cli.utils.helpers import print_generator_in_chunks
from cnvrgv2.config import error_messages
from cnvrgv2.errors import CnvrgError
from cnvrgv2.utils.validators import validate_volume_title


@click.group(name='volume')
def volume_group():
    pass


@volume_group.command()
@click.option('-t', '--title', prompt=messages.VOLUME_TITLE_PROMPT, help=messages.VOLUME_HELP_TITLE)
@click.option('-si', '--size', prompt=messages.VOLUME_SIZE_PROMPT, help=messages.VOLUME_HELP_SIZE)
@click.option('-c', '--cluster', prompt=messages.CLUSTER_SLUG_PROMPT, help=messages.VOLUME_HELP_CLUSTER_SLUG)
@click.option('-sc', '--storage-class-slug', prompt=messages.STORAGE_CLASS_PROMPT_SLUG,
              help=messages.STORAGE_CLASS_HELP_SLUG)
@click.option('-rwm', '--read-write-many', prompt=messages.READ_WRITE_MANY_PROMPT, default=True,
              help=messages.VOLUME_HELP_RWM)
@prepare_command()
def create(cnvrg, logger, title=None, size=None, cluster=None, storage_class_slug=None, read_write_many=True):
    """
    Create volume
    """
    if not validate_volume_title(title):
        logger.log_and_echo(error_messages.INVALID_VOLUME_TITLE.format(title), error=True)
    logger.log_and_echo(messages.VOLUME_CREATING_MESSAGE)
    new_volume = cnvrg.volumes.create(title=title, size=size, cluster=cluster, storage_class=storage_class_slug,
                                      read_write_many=read_write_many)
    success_message = messages.VOLUME_CREATED_SUCCESSFULLY.format(new_volume.title, new_volume.slug)
    logger.log_and_echo(success_message)


@volume_group.command()
@click.option('-t', '--title', prompt=messages.VOLUME_TITLE_PROMPT, help=messages.VOLUME_HELP_TITLE)
@click.option('-pvc', '--pvc-name', prompt=messages.PVC_CLAIM_NAME_PROMPT_TITLE,
              help=messages.PVC_CLAIM_NAME_PROMPT_TITLE)
@click.option('-sc', '--storage-class-slug', prompt=messages.STORAGE_CLASS_PROMPT_SLUG,
              help=messages.STORAGE_CLASS_HELP_SLUG)
@prepare_command()
def connect(cnvrg, logger, title=None, pvc_name=None, storage_class_slug=None):
    """
    Connect Storage Class
    """
    if not validate_volume_title(title):
        logger.log_and_echo(error_messages.INVALID_VOLUME_TITLE.format(title), error=True)

    logger.log_and_echo(messages.VOLUME_CONNECT_MESSAGE)
    connected_volume = cnvrg.volumes.connect(title=title, pvc_name=pvc_name, storage_class=storage_class_slug)
    success_message = messages.VOLUME_CONNECT_SUCCESSFULLY.format(connected_volume.title, connected_volume.slug)
    logger.log_and_echo(success_message)


@volume_group.command()
@click.option('-s', '--slug', help=messages.VOLUME_HELP_SLUG)
@prepare_command()
def disconnect(cnvrg, logger, slug):
    """
    discconect a volume
    """
    try:
        volume = cnvrg.volumes.get(slug=slug)
    except CnvrgError as error:
        logger.log_and_echo(message=str(error), error=True)
        return
    success_message = messages.VOLUME_DISCONNECTED_MESSAGE_SUCCESSFULLY.format(volume.title)
    volume.disconnect()
    logger.log_and_echo(success_message)


@volume_group.command()
@click.option('-s', '--slug', help=messages.VOLUME_HELP_SLUG)
@prepare_command()
def delete(cnvrg, logger, slug):
    """
    Deletes a volume
    """
    try:
        volume = cnvrg.volumes.get(slug=slug)
    except CnvrgError as error:
        logger.log_and_echo(message=str(error), error=True)
        return
    success_message = messages.VOLUME_DELETE_MESSAGE_SUCCESSFULLY.format(volume.title)
    volume.delete()
    logger.log_and_echo(success_message)


@volume_group.command(name='list')
@click.option('-p', '--page-size', default=20, help=messages.PAGE_SIZE_HELP)
@click.option('-s', '--sort', default="-id", help=messages.SORT_HELP)
@click.option('-cs', '--chunk-size', default=20, type=int, help='Number of volumes to display at a time')
@click.option('-l', '--limit', type=int, help='Maximum number of volumes to display')
@prepare_command()
def list_command(cnvrg, logger, sort, limit, chunk_size, page_size):
    """
    List all volumes
    """
    try:
        volumes = cnvrg.volumes.list(sort=sort, page_size=page_size)
    except CnvrgError as error:
        logger.log_and_echo(message=str(error), error=True)
        return
    attributes_field_name = ["Title", "Volume Slug", "Size", "Storage Class Slug", "Access Mode", "Status"]
    attributes = ["title", "slug", "total_space", "storage_class_slug", "access_mode", "status"]
    print_generator_in_chunks(
        generator=volumes,
        chunk_size=chunk_size,
        limit=limit,
        object_attributes=attributes,
        attributes_field_name=attributes_field_name,
        callback=volume_row_callback
    )


def volume_row_callback(row):
    """
    Custom callback to format the size from KB to GB
    """
    size_in_bytes = row[2]
    size_in_gb = size_in_bytes / (1024 ** 3)
    row[2] = f"{size_in_gb:.2f} GB"
    return row


@volume_group.command(name='get')
@click.option('-s', '--slug', help=messages.VOLUME_HELP_SLUG)
@prepare_command()
def get(cnvrg, logger, slug):
    """
    Get a specific volume by slug
    """
    try:
        volume = cnvrg.volumes.get(slug=slug)
    except CnvrgError as error:
        logger.log_and_echo(message=str(error), error=True)
        return

    attributes_field_name = ["Title", "Volume Slug", "Size", "Storage Class Slug", "Access Mode", "Status"]
    attributes = ["title", "slug", "total_space", "storage_class_slug", "access_mode", "status"]
    print_generator_in_chunks(
        generator=iter([volume]),
        chunk_size=1,
        limit=1,
        object_attributes=attributes,
        attributes_field_name=attributes_field_name,
        callback=volume_row_callback
    )



import codecs
import locale
import os
import sys

import click
from cnvrgv2._version import __version__

from cnvrgv2.cli.logger.logger import CnvrgLogger
from cnvrgv2.cli.modules.config import config
from cnvrgv2.cli.modules.dataset.dataset import dataset_group
from cnvrgv2.cli.modules.images.images import image_group
from cnvrgv2.cli.modules.libraries.library import library_group
from cnvrgv2.cli.modules.logs.logs import logs
from cnvrgv2.cli.modules.members.members import members_group
from cnvrgv2.cli.modules.labels.labels import labels_group
from cnvrgv2.cli.modules.organization_settings.organization_settings import organization_settings_group
from cnvrgv2.cli.modules.project.endpoint import endpoint_group
from cnvrgv2.cli.modules.project.experiment import experiment_group
from cnvrgv2.cli.modules.project.project import project_group
from cnvrgv2.cli.modules.project.workspace import workspace_group
from cnvrgv2.cli.modules.project.webapp import webapp_group
from cnvrgv2.cli.modules.project.flows import flow_group
from cnvrgv2.cli.modules.registries.registries import registry_group
from cnvrgv2.cli.modules.ssh.ssh import ssh_group
from cnvrgv2.cli.modules.storage_class.storage_class import storage_class_group
from cnvrgv2.cli.modules.datasource.datasource import datasource_group
from cnvrgv2.cli.modules.users import users
from cnvrgv2.cli.modules.volumes.volume import volume_group
from cnvrgv2.cli.utils import messages
from cnvrgv2.errors import CnvrgLoginError


"""
To avoid issues reading unicode chars from stdin or writing to stdout, we need to ensure that the
python3 runtime is correctly configured, if not, we try to force to utf-8.
"""
if codecs.lookup(locale.getpreferredencoding()).name == 'ascii':
    utf_8_locales = set([loc for loc in locale.locale_alias.values() if loc.lower().endswith((".utf-8", ".utf8"))])
    english_locales = [loc for loc in utf_8_locales if loc.lower().startswith('en_us')]

    if len(english_locales) > 0:
        os.environ['LANG'] = english_locales[0]
        os.environ['LC_ALL'] = english_locales[0]

    if codecs.lookup(locale.getpreferredencoding()).name == 'ascii':
        print("Could not set locale variable automatically.")
        print("Please set the LANG and LC_ALL env variables to a proper value before calling this script.")
        print("The following suitable locales were discovered:")
        print(', '.join('{}'.format(k) for k in utf_8_locales))
        sys.exit(-1)


@click.group()
@click.version_option(__version__, "-v", "--version",
                      message="%(prog)s %(version)s")
def entry_point():
    pass


def safe_entry_point():
    try:
        entry_point()
    except CnvrgLoginError as e:
        click.echo(str(e), err=True)
        exit(1)
    except Exception as e:
        click.echo(messages.CLI_UNEXPECTED_ERROR.format(str(e)), err=True)
        exit(1)


entry_point.add_command(logs)
entry_point.add_command(config)
entry_point.add_command(ssh_group)
entry_point.add_command(image_group)
entry_point.add_command(users.login)
entry_point.add_command(users.logout)
entry_point.add_command(users.me)
entry_point.add_command(flow_group)
entry_point.add_command(project_group)
entry_point.add_command(library_group)
entry_point.add_command(members_group)
entry_point.add_command(labels_group)
entry_point.add_command(dataset_group)
entry_point.add_command(endpoint_group)
entry_point.add_command(registry_group)
entry_point.add_command(workspace_group)
entry_point.add_command(experiment_group)
entry_point.add_command(webapp_group)
entry_point.add_command(organization_settings_group)
entry_point.add_command(CnvrgLogger.set_logs_keep_duration)
entry_point.add_command(storage_class_group)
entry_point.add_command(volume_group)
entry_point.add_command(datasource_group)
